
#include <linux/config.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/init.h>
#include <linux/timex.h>
#include <linux/types.h>
#include <asm/msr.h>

MODULE_LICENSE("GPL");

#define BYTES_PER_WORD          sizeof(void *)

#define NALLOC 1000

static struct ctest {
	kmem_cache_t *c;
	char name[16];
	void *pointers[NALLOC];
} testdata[20];

static void testalloc(int size)
{
	static struct ctest *t = testdata;
	kmem_cache_t *c;
	int free0, free1, i;
	struct sysinfo info;
	u32 tsc0, tsc1;

	if (size < BYTES_PER_WORD) return;
	if (size > (1<<5)*PAGE_SIZE) return;
	sprintf(t->name, "test-%i", size);
	si_meminfo(&info);
	free0 = info.freeram;
	rdtscl(tsc0);
	c = kmem_cache_create(t->name, size, 0,
			      0, NULL, NULL);

	for (i=0; i<NALLOC; i++)
		t->pointers[i] = kmem_cache_alloc(c, GFP_KERNEL);
	rdtscl(tsc1);
	si_meminfo(&info);
	free1 = info.freeram;
	t->c = c;
	t++;
	printk("size %6i: free was %6i, now %6i (delta %6i)"
	       " time %4lius\n", size, free0, free1,
	       free1-free0, (tsc1-tsc0)/(cpu_khz/1000));
}

int cmt_init(void)
{
	int sizes[]={1,2,5,0};
	struct sysinfo info;
	int i, j;
	
	si_meminfo(&info);
	printk("free %li (unit %i)\n",
	       info.freeram, info.mem_unit);
	for(i = 1; i <= 100*1000; i*=10)
		for (j=0; sizes[j]; j++)
			testalloc(i*sizes[j]);
	return 0;
}

void cmt_exit(void)
{
	struct ctest *t;
	int i;

	/* free them all */
	for (t = testdata; t->c; t++) {
		for (i=0; i<NALLOC; i++)
			kmem_cache_free(t->c, t->pointers[i]);
		kmem_cache_destroy(t->c);
	}

}



module_init(cmt_init);
module_exit(cmt_exit);

